use futures::TryFutureExt;
use tokio::try_join;

use crate::{change_state, prelude::*};

#[inline]
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    name: String,
    _: String,
) -> crate::Result<()> {
    dialogue.exit().await?;

    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    Account::delete_by_id((user_id, name)).exec(&db).await?;

    ids.alter_message(
        &bot,
        "The account is successfully deleted",
        deletion_markup(),
        None,
    )
    .await?;

    Ok(())
}

#[inline]
pub async fn delete(
    bot: Throttle<Bot>,
    q: CallbackQuery,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    (hash, is_command): (super::NameHash, bool),
) -> crate::Result<()> {
    let mut ids: MessageIds = q.message.as_ref().unwrap().into();
    let user_id = q.from.id.0;

    let Some(name) = name_from_hash(&db, user_id, &hash).await? else {
        try_join!(
            bot.send_message(ids.0, "Account wasn't found")
                .reply_markup(deletion_markup())
                .send(),
            bot.answer_callback_query(q.id).send()
        )?;

        return Ok(());
    };

    let response = async {
        const TEXT: &str = "Send master password. \
            Once you send the master password the account is unrecoverable";

        if is_command {
            ids.alter_message(&bot, TEXT, None, None).await?;
        } else {
            let msg = bot.send_message(ids.0, TEXT).await?;
            ids = MessageIds::from(&msg);
        };

        Ok::<_, crate::Error>(())
    };

    try_join!(response, bot.answer_callback_query(q.id).send().err_into())?;

    change_state!(dialogue, ids, (name), State::GetMasterPass, get_master_pass);

    Ok(())
}
